GET_PROVIDER_LIST_V2 = "v2/llm/providers"	 # GET -> Gets a list of available LLM providers.
GET_PROVIDER_DATA_V2 = "v2/llm/providers/{providerName}"  # GET -> Gets specific data for a provider.
GET_PROVIDER_MODELS_V2 = "v2/llm/providers/{providerName}/models"  # GET -> Gets a list of models for a specific provider.
GET_MODEL_DATA_V2 = "v2/llm/providers/{providerName}/models/{modelNameOrId}"  # GET -> Gets data for a specific model.